﻿using System.Collections.Generic;
using MRGSP.SurveyMaster.Core;
using MRGSP.SurveyMaster.Core.Model;
using MRGSP.SurveyMaster.Core.Repository;

namespace MRGSP.SurveyMaster.Data
{
    public class Repo<T> : IRepo<T> where T : new()
    {
        protected string Cs;

        public Repo(IConnectionFactory connFactory)
        {
            Cs = connFactory.GetConnectionString();
        }

        public T Get(long id)
        {
            return DbUtil.Get<T>(id, Cs);
        }

        public IEnumerable<T> GetAll()
        {
            return DbUtil.GetAll<T>(Cs);
        }

        //returns the new autogenerated id
        public virtual long Insert(T o)
        {
            return DbUtil.Insert(o, Cs);
        }

        public virtual int Update(T o)
        {
            return DbUtil.Update(o, Cs);
        }

        public virtual int UpdateWhatWhere(object what, object where)
        {
            return DbUtil.UpdateWhatWhere<T>(what, where, Cs);
        }

        //returns rows affected
        public virtual int InsertNoIdentity(T o)
        {
            return DbUtil.InsertNoIdentity(o, Cs);
        }

        public IEnumerable<T> GetPage(int page, int pageSize)
        {
            return DbUtil.GetPage<T>(page, pageSize, Cs);
        }

        public int Count()
        {
            return DbUtil.Count<T>(Cs);
        }

        public IPageable<T> GetPageable(int page, int pageSize)
        {
            return new Pageable<T>
                       {
                           Page = GetPage(page, pageSize),
                           PageCount = Tools.GetPageCount(pageSize, Count()),
                           PageIndex = page,
                       };
        }

        // example: repo.GetWhere(new { FirstName ="jhon", LastName="philips"});
        public IEnumerable<T> GetWhere(object where)
        {
            return DbUtil.GetWhere<T>(where, Cs);
        }

        public virtual int Delete(long id)
        {
            return DbUtil.Delete<T>(id, Cs);
        }

        public int CountWhere(object where)
        {
            return DbUtil.CountWhere<T>(where, Cs);
        }
    }
}