﻿using System.Collections.Generic;
using MRGSP.SurveyMaster.Core;
using MRGSP.SurveyMaster.Core.Model;
using MRGSP.SurveyMaster.Core.Service.Utils;

namespace MRGSP.SurveyMaster.Service.Utils
{
    public static class QuestionCreateBuilder
    {
        private static readonly IDictionary<ControlType, IQuestionBuilder> Implementations
            = new Dictionary<ControlType, IQuestionBuilder>
                  {
                      { ControlType.Checkbox, new ListCreateBuilder() },
                      { ControlType.Textbox, new ListCreateBuilder() },
                      { ControlType.TextArea, new ListCreateBuilder() },
                      { ControlType.Radio, new ListCreateBuilder() },
                      { ControlType.Header, new HeaderCreateBuilder() },
                      { ControlType.Grid, new GridCreateBuilder()},
                      { ControlType.GridRow, new GridRowCreateBuilder()},
                      { ControlType.GridCell, new GridCellCreateBuilder()},
                      { ControlType.GridCellList, new GridCellListCreateBuilder()},
                  };

        public static IQuestionBuilder Get(ControlType type)
        {
            if (Implementations.ContainsKey(type))
                return Implementations[type];
            return Implementations[ControlType.Checkbox];
        }

        public static string Build(Question question)
        {
            return Get(question.Type).Build(question);
        }
    }
}